﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Windows.Forms;
using Microsoft.VisualBasic.FileIO;

namespace CShp_ArquivosDuplicados
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Cria uma instância do serviço do provedor de criptografia MD5
        private MD5 Md5 = MD5.Create();
        private bool Deleting = false;

        private void btnDiretorio_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd1 = new FolderBrowserDialog();

            fbd1.SelectedPath = txtDiretorio.Text;

            if (fbd1.ShowDialog() == DialogResult.OK)
                txtDiretorio.Text = fbd1.SelectedPath;
        }

        private void btnProcurar_Click(object sender, EventArgs e)
        {
            tvArquivos.Visible = false;
            tvArquivos.Nodes.Clear();
            lblNumeroArquivosDuplicados.Text = "";
            Refresh();

            try
            {
                // Obtem uma lista de arquivos e seus valores de hash
                var get_info =
                    from string _nomeArquivo in Directory.GetFiles(txtDiretorio.Text)
                    select new
                    {
                        Name = _nomeArquivo,
                        Hash = BytesToString(GetHash(_nomeArquivo))
                    };

                // Agrupa os arquivos por valor de hash
                var group_infos =
                    from info in get_info
                    group info by info.Hash into g
                    where g.Count() > 1
                    //orderby g.Key
                    select g;

                // percore os arquivos
                int num_groups = 0;
                int num_arquivos = 0;
                foreach (var g in group_infos)
                {
                    num_groups++;
                    TreeNode hash_node = tvArquivos.Nodes.Add(g.Key.ToString());
                    foreach (var info in g)
                    {
                        num_arquivos++;
                        TreeNode arquivo_node = new TreeNode(info.Name);
                        arquivo_node.Tag = new FileInfo(info.Name);
                        hash_node.Nodes.Add(arquivo_node);
                    }
                }

                // Exibe o numero de arquivos duplicados
                lblNumeroArquivosDuplicados.Text = (num_arquivos - num_groups).ToString() + " arquivos duplicados";

                // Expande os nodes.
                tvArquivos.ExpandAll();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                // Cai para o topo
                if (tvArquivos.Nodes.Count > 0)
                    tvArquivos.Nodes[0].EnsureVisible();
                tvArquivos.Visible = true;

                Cursor = Cursors.Default;
            }
        }

        // Retorna o codigo hash como uma string
        private string BytesToString(byte[] bytes)
        {
            string resultado = "";
            foreach (byte b in bytes) resultado += b.ToString("x2");
            return resultado;
        }

        // Calcula o hash do arquivo
        private byte[] GetHash(string _nomeArquivo)
        {
            using (FileStream stream = File.OpenRead(_nomeArquivo))
            {
                return Md5.ComputeHash(stream);
            }
        }

        private void btnSelecionarDuplicados_Click(object sender, EventArgs e)
        {
            foreach (TreeNode hash_node in tvArquivos.Nodes)
            {
                hash_node.Checked = false;
                hash_node.Nodes[0].Checked = false;
                for (int i = 1; i < hash_node.Nodes.Count; i++)
                    hash_node.Nodes[i].Checked = true;
            }
        }

        private void DeletarDuplicados_Click(object sender, EventArgs e)
        {
            Deleting = true;
            tvArquivos.Visible = false;
            lblNumeroArquivosDuplicados.Text = "";
            Refresh();

            try
            {
                // Faz uma lista de nodes para deletar
                List<TreeNode> nodes_para_deletar = new List<TreeNode>();
                foreach (TreeNode hash_node in tvArquivos.Nodes)
                {
                    foreach (TreeNode arquivo_node in hash_node.Nodes)
                    {
                        // Verifica se o arquivo do node esta marcado(checked.)
                        if (arquivo_node.Checked) nodes_para_deletar.Add(arquivo_node);
                    }
                }

                // Deleta os nodes selecionados e seus arquivos
                foreach (TreeNode arquivo_node in nodes_para_deletar)
                {
                    // Obtem o FileInfo da propriedadae Tag
                    FileInfo file_info = arquivo_node.Tag as FileInfo;

                    // Move o arquivod para a lixeira
                    DeletarArquivo(file_info.FullName);

                    // Remove o arquivo do TreeView.
                    arquivo_node.Remove();
                }

                // Faz uma lista do tamanho dos nodes com os filhos restantes
                nodes_para_deletar = new List<TreeNode>();
                foreach (TreeNode hash_node in tvArquivos.Nodes)
                {
                    if (hash_node.Nodes.Count == 0) nodes_para_deletar.Add(hash_node);
                }

                // Deleta o tamanho dos nodes com os filhos
                foreach (TreeNode hash_node in nodes_para_deletar)
                {
                    hash_node.Remove();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                // Rola para o topo 
                if (tvArquivos.Nodes.Count > 0)
                    tvArquivos.Nodes[0].EnsureVisible();

                tvArquivos.Visible = true;
                Deleting = false;
            }
        }

        // Deleta um arquivo ou move o arquivo para lixeira
        public static void DeletarArquivo(string nomeArquivo)
        {
            DeletarArquivo(nomeArquivo, false, false);
        }
        public static void DeletarArquivo(string nomeArquivo, bool confirma)
        {
            DeletarArquivo(nomeArquivo, confirma, false);
        }
        public static void DeletarArquivo(string nomeArquivo, bool confirma, bool deletar_permanente)
        {
            UIOption ui_option = UIOption.OnlyErrorDialogs;
            if (confirma) ui_option = UIOption.AllDialogs;

            RecycleOption recycle_option = recycle_option = RecycleOption.SendToRecycleBin;

            if (deletar_permanente)
                recycle_option = RecycleOption.DeletePermanently;

            try
            {
                FileSystem.DeleteFile(nomeArquivo, ui_option, recycle_option);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Erro ao deletar o arquivo .\n" + ex.Message,"Error", MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
            }
        }

        private void btnSair_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
